(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global.Karin = {})));
}(this, (function (exports) {
  function makeUrl(chunks, interpolations, pastLast) {
    if ( pastLast === void 0 ) pastLast = false;

    var str = "";
    var lastIndex = 0;

    if (pastLast) {
      lastIndex = interpolations.length - 1;
    }

    for (var i = 0; i < chunks.length; i++) {
      var key = chunks[i];
      var value = interpolations[i];

      if (pastLast && i === lastIndex) {
        str += key;
        continue;
      }

      if (value !== undefined) {
        if (typeof value === "object") {
          if (key[key.length - 1] === "?") {
            var generate = generateFromObject(value, "=", "&");
            str += "" + key + generate;
          } else {
            var generateParams = generateFromObject(value, "/", "/");
            str += "" + key + generateParams;
          }

          continue;
        }

        value += "";
      } else {
        str += key;
        continue;
      }

      value += "";

      if (/[a-z]/.test(key[key.length - 1])) {
        str += key + "/" + value;
      } else {
        str += "" + key + value;
      }
    }

    return str;
  }

  function getForceOptions(url, cb) {
    return url.replace(/\s--([^\s]+)?/g, function (a, b) {
      cb(b);
      return "";
    });
  }

  function generateFromObject(obj, join1, join2) {
    return Object.keys(obj).map(function (prop) {
      return [prop, obj[prop]].map(encodeURIComponent).join(join1);
    }).join(join2);
  }

  function headersToObject(string) {
    var lines = string.split("\n");
    var headers = {};

    for (var i = 0; i < lines.length; i++) {
      var value = lines[i];
      var iOf = value.indexOf(": ");

      if (iOf !== -1) {
        headers[value.slice(0, iOf)] = value.slice(iOf + 2);
      }
    }

    return headers;
  }

  function dataBody(contentType, encode, raw) {
    if (!encode) {
      if (/^application\/json/.test(contentType)) {
        return JSON.parse(raw);
      }
    } else {
      var final = null;

      switch (encode) {
        case "json":
          final = JSON.parse(raw);
          break;

        case "text":
          final = raw.toString();
          break;

        case "raw":
          final = raw;
          break;
      }

      return final;
    }
  }

  function xhrGet(url, params) {
    return new Promise(function (resolve, reject) {
      var xhr = new XMLHttpRequest();
      xhr.open("GET", url, true);
      var entries = Object.entries(params.headers);

      if (entries.length > 0) {
        for (var i = 0, list = Object.entries(params.headers); i < list.length; i += 1) {
          var ref = list[i];
          var key = ref[0];
          var value = ref[1];

          xhr.setRequestHeader(key, value);
        }
      }

      xhr.onload = function () {
        var headers = headersToObject(xhr.getAllResponseHeaders().toLowerCase());
        var contentType = headers["content-type"];
        resolve({
          headers: headers,
          message: this.statusText,
          status: this.status,
          data: dataBody(contentType, params.encode, this.response)
        });
      };

      xhr.onerror = function () {
        reject(new TypeError("Network request failed"));
      };

      if ("timeout" in params) {
        xhr.timeout = params.timeout;

        xhr.ontimeout = function () {
          reject(new TypeError("Network request failed"));
        };
      }

      xhr.send();
    });
  }

  function get(param) {
    var keys = [], len = arguments.length - 1;
    while ( len-- > 0 ) keys[ len ] = arguments[ len + 1 ];

    var callOrReturn = null;

    if (param.constructor === Object) {
      callOrReturn = true;
    } else {
      callOrReturn = false;
    }

    function t() {
      var chunks = callOrReturn ? arguments[0] : param;
      var interpolations = callOrReturn ? [].slice.call(arguments, 1) : keys;
      var settings = callOrReturn ? param : {
        headers: {},
        encode: ""
      };
      var options = Object.assign({}, {headers: Object.assign({}, settings.headers),
        encode: ""},
        settings);
      var str = makeUrl(chunks, interpolations);
      var normalizeUrl = getForceOptions(str, function (b) {
        if (b === "json") {
          options.encode = "json";
        }

        if (b === "raw") {
          options.encode = "raw";
        }
      });

      if (options.origin) {
        normalizeUrl = "" + (options.origin) + normalizeUrl;
      }

      return xhrGet(normalizeUrl, options);
    }

    if (callOrReturn) {
      return t;
    } else {
      return t();
    }
  }

  function xhrPost(url, params, postData) {
    return new Promise(function (resolve, reject) {
      var xhr = new XMLHttpRequest();
      xhr.open("POST", url, true);
      var entries = Object.entries(params.headers);

      if (entries.length > 0) {
        for (var i = 0, list = Object.entries(params.headers); i < list.length; i += 1) {
          var ref = list[i];
          var key = ref[0];
          var value = ref[1];

          xhr.setRequestHeader(key, value);
        }
      }

      xhr.onload = function () {
        var headers = headersToObject(xhr.getAllResponseHeaders().toLowerCase());
        var contentType = headers["content-type"];
        resolve({
          message: xhr.statusText,
          status: xhr.status,
          headers: headers,
          data: dataBody(contentType, params.encode, xhr.response)
        });
      };

      xhr.onerror = function () {
        reject(new TypeError("Network request failed"));
      };

      if ("timeout" in params) {
        xhr.timeout = params.timeout;

        xhr.ontimeout = function () {
          reject(new TypeError("Network request failed"));
        };
      }

      xhr.send(postData);
    });
  }

  function post(param) {
    var keys = [], len = arguments.length - 1;
    while ( len-- > 0 ) keys[ len ] = arguments[ len + 1 ];

    var callOrReturn = null;

    if (param.constructor === Object) {
      callOrReturn = true;
    } else {
      callOrReturn = false;
    }

    function t() {
      var chunks = callOrReturn ? arguments[0] : param;
      var interpolations = callOrReturn ? [].slice.call(arguments, 1) : keys;
      var settings = callOrReturn ? param : {
        headers: {},
        encode: ""
      };
      var options = Object.assign({}, {headers: Object.assign({}, settings.headers),
        encode: ""},
        settings);
      var str = makeUrl(chunks, interpolations, true);
      var postData = interpolations[interpolations.length - 1];

      if (typeof postData === "object") {
        postData = JSON.stringify(postData);
        options.headers["Content-Type"] = "application/json";
      }

      var normalizeUrl = getForceOptions(str, function (b) {
        if (b === "json") {
          options.encode = "json";
        }

        if (b === "raw") {
          options.encode = "raw";
        }
      });

      if (options.origin) {
        normalizeUrl = "" + (options.origin) + normalizeUrl;
      }

      return xhrPost(normalizeUrl, options, postData);
    }

    if (callOrReturn) {
      return t;
    } else {
      return t();
    }
  }

  var karinUtil = {
    get: null,
    post: null,

    setGetPost: function setGetPost(get, post) {
      this.get = get;
      this.post = post;
    },

    create: function create(params) {
      return {
        get: this.get(params),
        post: this.post(params)
      };
    }

  };

  karinUtil.setGetPost(get, post);
  var karin = karinUtil;

  exports.get = get;
  exports.post = post;
  exports.karin = karin;

})));
