var url = require('url');
var http = require('http');
var https = require('https');

function makeUrl(chunks, interpolations, pastLast) {
  if ( pastLast === void 0 ) pastLast = false;

  var str = "";
  var lastIndex = 0;

  if (pastLast) {
    lastIndex = interpolations.length - 1;
  }

  for (var i = 0; i < chunks.length; i++) {
    var key = chunks[i];
    var value = interpolations[i];

    if (pastLast && i === lastIndex) {
      str += key;
      continue;
    }

    if (value !== undefined) {
      if (typeof value === "object") {
        if (key[key.length - 1] === "?") {
          var generate = generateFromObject(value, "=", "&");
          str += "" + key + generate;
        } else {
          var generateParams = generateFromObject(value, "/", "/");
          str += "" + key + generateParams;
        }

        continue;
      }

      value += "";
    } else {
      str += key;
      continue;
    }

    value += "";

    if (/[a-z]/.test(key[key.length - 1])) {
      str += key + "/" + value;
    } else {
      str += "" + key + value;
    }
  }

  return str;
}

function getForceOptions(url$$1, cb) {
  return url$$1.replace(/\s--([^\s]+)?/g, function (a, b) {
    cb(b);
    return "";
  });
}

function generateFromObject(obj, join1, join2) {
  return Object.keys(obj).map(function (prop) {
    return [prop, obj[prop]].map(encodeURIComponent).join(join1);
  }).join(join2);
}

function dataBody(contentType, encode, raw) {
  if (!encode) {
    if (/^application\/json/.test(contentType)) {
      return JSON.parse(raw);
    }
  } else {
    var final = null;

    switch (encode) {
      case "json":
        final = JSON.parse(raw);
        break;

      case "text":
        final = raw.toString();
        break;

      case "raw":
        final = raw;
        break;
    }

    return final;
  }
}

function get(url$$1, params, parse) {
  var protocol = parse.protocol === "https:" ? https : http;
  return new Promise(function (resolve, reject) {
    var options = {
      hostname: parse.hostname,
      path: parse.path,
      headers: Object.assign({}, params.headers)
    };
    var req = protocol.get(options, function (res) {
      res.setEncoding("utf8");
      var raw = "";
      res.on("data", function (chunk) {
        raw += chunk;
      });
      res.on("end", function () {
        var contentType = res.headers["content-type"];
        resolve({
          message: res.statusMessage,
          status: res.statusCode,
          headers: res.headers,
          data: dataBody(contentType, params.encode, raw)
        });
      });
    });

    if ("timeout" in params) {
      req.setTimeout(params.timeout, function () {
        req.abort();
      });
    }

    req.on("error", function (err) {
      if (err.code === "ECONNRESET") {
        reject(new TypeError("Timeout occurs"));
      }

      reject(err);
    });
  });
}

function get$1(param) {
  var keys = [], len = arguments.length - 1;
  while ( len-- > 0 ) keys[ len ] = arguments[ len + 1 ];

  var callOrReturn = null;

  if (param.constructor === Object) {
    callOrReturn = true;
  } else {
    callOrReturn = false;
  }

  function t() {
    var chunks = callOrReturn ? arguments[0] : param;
    var interpolations = callOrReturn ? [].slice.call(arguments, 1) : keys;
    var settings = callOrReturn ? param : {
      headers: {},
      encode: ""
    };
    var options = Object.assign({}, {headers: Object.assign({}, settings.headers),
      encode: ""},
      settings);
    var str = makeUrl(chunks, interpolations);
    var normalizeUrl = getForceOptions(str, function (b) {
      if (b === "json") {
        options.encode = "json";
      }

      if (b === "raw") {
        options.encode = "raw";
      }
    });

    if (options.origin) {
      normalizeUrl = "" + (options.origin) + normalizeUrl;
    }

    return get(normalizeUrl, options, url.parse(normalizeUrl));
  }

  if (callOrReturn) {
    return t;
  } else {
    return t();
  }
}

var karinUtil = {
  get: null,
  post: null,

  setGetPost: function setGetPost(get, post) {
    this.get = get;
    this.post = post;
  },

  create: function create(params) {
    return {
      get: this.get(params),
      post: this.post(params)
    };
  }

};

karinUtil.setGetPost(get$1, function () { return console.warn('Http requests are temporarily not supported.'); });
var karin = karinUtil;

exports.get = get$1;
exports.karin = karin;
